
program semb_dips
  use sep
  implicit none
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! program !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  integer                       :: n1, n2, n3
  integer                       :: i1, i2, i3
  real   , allocatable          :: stack_in(:,:,:), stack_intrp(:,:,:)
  complex, allocatable          :: dips_out(:,:,:)
  real   , allocatable          :: dips_real(:,:,:)
  real                          :: xdip, ydip
  integer                       :: i1inc, i2inc, di1, di2, oi1, oi2, rad, smbsmp
  
  i1inc = 1
  i2inc = 1
  rad = 7!15
  smbsmp = 3
  
  call sep_init()
  call from_history( n1, n2, n3)
  call to_history  ( "n1", n1)
  call to_history  ( "n2", n2)
  call to_history  ( "n3", 1)
  call to_history  ( "esize", 8)
  
  call sep_close()
  
  allocate(stack_in(n1,n2,n3))
  allocate(dips_out(n1,n2,n3))
  allocate(dips_real(n1,n2,n3))
  
  allocate(stack_intrp(n1*10, 2*rad+1, 2*rad+1))
  
  dips_out = 0
  
  call sep_read( stack_in )
  
  call omp_set_num_threads(15)
  
  do i3=200,200
      
    do i2=1,n2,i2inc
      write(0,*)'dips for il ',i3,'xl',i2
    
      call interp_data(n1, i2, i3, rad, stack_in, stack_intrp)
!$omp parallel do private(xdip, ydip)
      do i1=1,n1,i1inc
        call dip_est(i1, rad, smbsmp, stack_intrp, xdip, ydip)
        
        do di1=-i1inc+1,i1inc-1
          oi1 = di1 + i1
          if (oi1 >= 1 .and. oi1 <= n1) &
            dips_out(oi1,i2,i3) = dips_out(oi1,i2,i3) + cmplx(xdip, ydip) * (i1inc-abs(di1)) / i1inc
        end do
        
      end do
!$omp end parallel do

      do di2=-i2inc+1,i2inc-1
        oi2 = di2 + i2
        if (oi2 >= 1 .and. oi2 <= n2 .and. .not. oi2 == i2) &
          dips_out(:,oi2,i3) = dips_out(:,oi2,i3) + dips_out(:,i2,i3) * (i2inc-abs(di2)) / i2inc
      end do
      
    end do
  end do
  
  dips_real = real(dips_out)
  call sep_write( dips_out(:,:,200) )
  write(0,*)'n1=',n1,'n2=',n2,'n3=',n3
  
  contains
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! subroutines !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  
  subroutine dip_est(i1, rad, smbsmp, stack_intrp, bxdip, bydip)
    integer, intent(in )     :: i1, rad
    real   , intent(in )     :: stack_intrp(:,:,:)
    real   , intent(out)     :: bxdip, bydip
    integer                  :: nxdip, nydip
    integer                  :: ixdip, iydip
    real                     :: mxdip, mydip
    real                     ::  xdip,  ydip
    real                     ::  dx,  dy
    integer                  :: idx, idy
    real                     :: di1
    integer                  :: ind, maxind, j, smbsmp
    real                     :: smp, enr(8), smm(8)
    real                     :: smb, maxsmb
    
    mxdip = 3!.3
    mydip = 3!.3
    nxdip = 60!40
    nydip = 60!40
    dx = 1.
    dy = 1.
    maxind = (size(stack_intrp,1)-8)*10-9
    maxsmb = 0.
    
    
    do ixdip= -nxdip, nxdip
      do iydip = -nydip, nydip
        xdip = mxdip*ixdip/nxdip
        ydip = mydip*iydip/nydip
        
        enr = 0.
        smm = 0.
        
        do idx = -rad, rad
          do idy = -rad, rad
        
            di1  = xdip*idx + ydip*idy
            ind = (i1 + di1 + smbsmp/2.)*10 - 0.5
            if (ind>=1 .AND. ind<=maxind) then
              do j=1,smbsmp
                smp    = stack_intrp(ind+10*j,rad+1+idx,rad+1+idy)
                smm(j) = smm(j) + smp
                enr(j) = enr(j) + smp**2
              end do
            end if
            
          end do
        end do
        
        smb = 0.
        do j=1,smbsmp
          if (enr(j) > 0) smb = smb + smm(j)**2 / enr(j)
        end do
          
        
        if (smb > maxsmb) then
          maxsmb = smb
          bxdip = xdip
          bydip = ydip
        end if
                
      end do
    end do
    
  end subroutine
  
  
  subroutine interp_data(n1, i2, i3, rad, stack_in, stack_intrp)
    integer, intent(in )                 :: n1, i2, i3, rad
    real   , intent(in )                 :: stack_in(:,:,:)
    real   , intent(out)                 :: stack_intrp(:,:,:)
    integer                              :: di2, di3
    real                                 :: tmp(n1)
    
    do di3=-rad,rad
      do di2=-rad,rad
        if (i3+di3 >= 1 .AND. i3+di3 <= size(stack_in, 3) .AND. &
            i2+di2 >= 1 .AND. i2+di2 <= size(stack_in, 2)) then
            call ovsamp( stack_in   (:,   i2+di2,   i3+di3), &
                         stack_intrp(:,rad+1+di2,rad+1+di3), &
                         n1                                  )
        end if
      end do
    end do
    
  end subroutine
      
  subroutine ovsamp(trace, outdat, nt)
    real   ,intent(in)  :: trace(:)
    real   ,intent(out) :: outdat(:)
    integer,intent(in)  :: nt
    
    integer                    :: i, j

    do i=1,nt-1
      do j=0,9
        outdat(i*10+j) = trace(i)*(10-j)/10 + trace(i+1)*j/10
      end do
    end do
    outdat(nt*10-9) = trace(nt)
  end subroutine
    
end program