
program vel_anal
  use sep3d_struct_mod
  use sep
  implicit none
  
  type (sep3d) :: input

  real   , allocatable              :: data(:,:), sums(:,:), sqrsums(:,:), panel(:,:,:), tmp(:), trcs(:,:), fomel_dips(:,:)
  complex, allocatable              :: semb_dips(:,:)
  integer, allocatable              :: sxs(:), sys(:), rxs(:), rys(:)
  integer                           :: n1, n2, n3, nh, i1, i2, i3, iv, nv, n, ntrcs, j
  integer                           :: err, l, min_sx, max_sx, min_sy, max_sy
  real                              :: off, v0, vm, v
  real                              :: tt, ttl, dx, dt, to, fsmp, val, frac
  real                              :: cx, cy
  integer                           :: ntyp, typ, ismp, jsmp, n2dip, dj, xl
  complex                           :: dip
  integer                           :: mode, want
  
  
  call sep_init()
  call init_3d()
  call init_sepf90() 
  call from_history( n1, n2, n3)
  
  nv = 141
  v0 = 1000
  vm = 8000
  dx = 20
  dt = 0.008
  n2dip=400
  ntyp=2
  ntrcs = 5000
  mode = 1
  dj = 3
  xl = 130
  
  if (mode == 0) then
    call to_history  ( "n2", 500)
    call to_history  ( "n3", 10)
  else
    call to_history  ( "n1", n1)
    call to_history  ( "n2", nv)
    call to_history  ( "o2", v0 )
    call to_history  ( "d2", (vm-v0)/(nv-1))
    call to_history  ( "label2", "velocity (m/s)")
    call to_history  ( "n3", ntyp)
    call to_history  ( "o3", 1 )
    call to_history  ( "d3", 1 )
  end if
  call init_sep3d("in",input,"INPUT")
  
  allocate(data      (n1   ,1)      )
  allocate(panel     (n1   ,nv,ntyp))
  allocate(semb_dips (n1   ,n2dip)  )
  allocate(fomel_dips(n1   ,n2dip)  )
  allocate(sums      (n1   ,10)     )
  allocate(sqrsums   (n1   ,10)     )
  allocate(sxs       (ntrcs)        ) 
  allocate(sys       (ntrcs)        ) 
  allocate(rxs       (ntrcs)        ) 
  allocate(rys       (ntrcs)        )
  allocate(trcs      (n1   ,ntrcs)  )
  
  call sep_read(semb_dips , 'semb_dips_cmplx.H')
  call sep_read(fomel_dips,'fomel_dips.H')
  
  cx = 3790+199*20     ! inline
  cy =  760+ (xl-1)*20 !  xline
  
  min_sx = 999999999
  min_sy = 999999999
  max_sx = -1
  max_sy = -1
  
  nh = 1
  n = 1
  l = 1
  do i3=1,n2
    call sep3d_grab_headers('in', input, nh, fwind=(/i3-1/),nwind=(/nh/))
    call sep3d_grab_key_vals(input,"sx"    ,sxs  (n:n)) 
    call sep3d_grab_key_vals(input,"sy"    ,sys  (n:n)) 
    call sep3d_grab_key_vals(input,"gx"    ,rxs  (n:n)) 
    call sep3d_grab_key_vals(input,"gy"    ,rys  (n:n)) 
    sxs(n) = sxs(n)/10000
    sys(n) = sys(n)/10000
    rxs(n) = rxs(n)/10000
    rys(n) = rys(n)/10000
    if (min_sx > sxs(n)) min_sx = sxs(n)
    if (min_sy > sys(n)) min_sy = sys(n)
    if (max_sx < sxs(n)) max_sx = sxs(n)
    if (max_sy < sys(n)) max_sy = sys(n)
    
    call keep_trc(sxs(n), sys(n), rxs(n), rys(n), cx, cy, want)
    if (want) then
      err = sep3d_read_data   ('in', input, data(:,:))
!      if (abs(sou_ys(n)/2+rec_ys(n)/2-cy)<1) data(:,:) = 0
      trcs(:,n) = data(:,1)
      n = n + 1
    end if
    
  end do
  n = n-1
  write(0,*)n, min_sx, max_sx, min_sy, max_sy
  
  if (mode == 0) call sep_write( trcs )
  
  if (mode == 1) then
    sums    = 0
    sqrsums = 0
    do typ=1,ntyp
      do iv=1,nv
        v = v0 + (iv-1)*(vm-v0)/(nv-1)
        sums    = 0
        sqrsums = 0
        do i2=1,n
          ttl = 0
          do i1=1,n1
            dip = semb_dips (i1,xl) * dt / dx
            to = (i1-1)*dt
            call comp_tt(typ, sxs(i2), sys(i2), rxs(i2), rys(i2), cx, cy, imag(dip), real(dip), to, tt)
            fsmp = tt/dt + 1
            ismp = fsmp
            frac            = fsmp - ismp 
            if (ismp >= 4 .and. ismp+4 <= n1 .and. (tt-ttl) > 0.7*dt) then
              do j=-dj,dj
                val             =       trcs(ismp+j  ,i2)*(1-frac)
                val             = val + trcs(ismp+j+1,i2)*frac
                sums   (i1,j+dj+1) = sums   (i1,j+dj+1) + val
                sqrsums(i1,j+dj+1) = sqrsums(i1,j+dj+1) + val**2
              end do
            end if
!            off = sqrt(1.*(sxs(i2)-rxs(i2))**2+(sys(i2)-rys(i2)))
!            if (tt-ttl < 0.7*dt) write(0,*)tt,off
            ttl = tt
            !if (i2==10) write(0,*)to,v,off,tt,fsmp
          end do ! times
        end do ! traces
        
        do i1=1,n1
          panel(i1,iv,typ) = sum( sums(i1,1:(2*dj+1))**2 / sqrsums(i1,1:(2*dj+1)) )
        end do
        
        if (mod(iv,10)==0) write(0,*)iv,'of',nv
      end do ! velocities
    end do ! typs
  
    do typ=1,ntyp
      do i1=1,n1
        panel(i1,:,typ) = panel(i1,:,typ) / maxval(panel(i1,:,typ))
        do iv=1,nv
          if (isnan(panel(i1,iv,typ))) panel(i1,iv,typ) = 0
        end do
      end do
    end do
  
    
    allocate(tmp(n1*nv*ntyp))
    tmp = reshape(panel, (/ n1*nv*ntyp /) )
    call sort(tmp, n1*nv*ntyp)
    panel = panel - tmp(n1*nv*ntyp/2)
    
    call sep_write( panel )
  end if !mode == 1
  
  contains
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  subroutine comp_tt(typ, sx, sy, rx, ry, cx, cy, xdip, ydip, to, tt)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    integer, intent(in )   :: typ, sx, sy, rx, ry
    real   , intent(in )   :: cx, cy, xdip, ydip, to   
    real   , intent(out)   :: tt
    real                   :: hx, hy, mx, my, dto

    hx  = (sx - rx) / 2.
    hy  = (sy - ry) / 2.
    mx  = (sx + rx) / 2. - cx
    my  = (sy + ry) / 2. - cy
    dto = xdip*mx + ydip*my
  
    if (typ==1) tt = sqrt(   to    **2 + 4*(hx**2+hy**2)/v**2 )
    if (typ==2) tt = sqrt( (to+dto)**2 + 4*(hx**2+hy**2)/v**2 )
    if (typ==3) tt = sqrt( (to+dto)**2 + 4*(hx**2+hy**2)/v**2 )
  
  end subroutine
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  subroutine keep_trc(sx, sy, rx, ry, cx, cy, want)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    integer, intent(in)         :: sx, sy, rx, ry
    real   , intent(in)         :: cx, cy
    integer, intent(out)        :: want
    real                        :: cmp_x, cmp_y, offst
    integer                     :: xw, yw
  
    xw = 3
    yw = 3
  
    cmp_x = (sx + rx) / 2.
    cmp_y = (sy + ry) / 2.
  
    want = cmp_x >= cx-xw*20-1 .and. cmp_x <= cx+xw*20+1 .and. &
           cmp_y >= cy-yw*20-1 .and. cmp_y <= cy+yw*20+1
         
!    offst = sqrt( 1.*(sx-rx)**2 + 1.*(sy-ry)**2)
!    want = cmp_x > cx-1 .and. cmp_x < cx+1 .and. offst < 200
  
  end subroutine
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   
   subroutine sort(d, max)
    real   , intent(inout) :: d(:)
    integer, intent(in)    :: max
    real                   :: x
    integer                :: m, k, i

    m = 1
20    if((2**m).gt.max) goto 30
    m = m+1
    goto 20
30    m = m-1
    m = 2**m
100    k = max-m
    do 500 i=1,k
       do 300 j=i,1,(-m)
          if(d(j+m).ge.d(j)) goto 500
          x = d(j)
          d(j) = d(j+m)
          d(j+m) = x
300       continue
500    continue
    m = m/2
    if(m.gt.0) goto 100

    return
    end subroutine
  
end program
